/**
 * @file flock.cpp
 * @brief Function definitions for flocks.
 *
 * @author Kristin Siu (kasiu)
 * @author Eric Butler (kasiu)
 */

#include "game/flock.hpp"

#define PARTICLE_SIZE 6

namespace _462 {

bool Flock::initialize( size_t num_boids, const Vector3& goal )
{
    boids.resize( num_boids );
    this->goal = goal;

    // TODO any other init code you need

    return true;
}

void Flock::set_boid( size_t index, const Vector3& position, const Vector3& velocity, real_t mass )
{
    assert( index < boids.size() );
    Boid& b = boids[index];
    b.position = position;
    b.velocity = velocity;
    b.mass = mass;
}


void Flock::set_goal( const Vector3& goal )
{
    this->goal = goal;
}

size_t Flock::size() const
{
    return PARTICLE_SIZE * boids.size();
}

void Flock::get_state( real_t* arr, real_t* time ) const
{
    assert( arr && time );
    size_t num_part = boids.size();

    for ( size_t i = 0; i < num_part; ++i ) {
        size_t idx = PARTICLE_SIZE * i;
        *(Vector3*)(arr+idx+0) = boids[i].position;
        *(Vector3*)(arr+idx+3) = boids[i].velocity;
    }

    *time = this->time;
}

void Flock::set_state( const real_t* arr, const real_t time )
{
    assert( arr );
    size_t num_part = boids.size();

    for ( size_t i = 0; i < num_part; ++i ) {
        size_t idx = PARTICLE_SIZE * i;
        boids[i].position = *(Vector3*)(arr+idx+0);
        boids[i].velocity = *(Vector3*)(arr+idx+3);
    }

    this->time = time;
}

/**
 * @brief Updates the flock by applying forces
 * @param dt The timestep
 * @bug unimplemented
 */
void Flock::eval_deriv( real_t* deriv_result )
{
    assert( deriv_result );
    size_t num_boids = boids.size();

    // zero out forces
    for( size_t i = 0; i < num_boids; i++ ) {
        boids[i].force_accumulator = Vector3::Zero;
    }

    // TODO apply forces

    // compute derivative
    for ( size_t i = 0; i < num_boids; ++i ) {
        size_t idx = PARTICLE_SIZE * i;
        const Boid& p = boids[i];
        *(Vector3*)(deriv_result+idx+0) = p.velocity;
        *(Vector3*)(deriv_result+idx+3) = p.force_accumulator / p.mass;
    }
}

void Flock::render() const
{
    // TODO replace this with more interesting render code, like models

    size_t num_boids = boids.size();
    glColor3f( 1.0f, 0.0f, 0.0f );

    for( size_t i = 0; i < num_boids; i++ ) {
        const Boid& b = boids[i];
        Vector3 p = b.position;

        glPushMatrix();
        glTranslated( p.x, p.y, p.z );
        glBegin( GL_POINTS );
        glVertex3f( 0.0f, 0.0f, 0.0f );
        glEnd();
        glPopMatrix();
    }
}

} /* _462 */

